<?php
// Own namespace
namespace CoreFramework\Container\Socket;

// Import framework stuff
use CoreFramework\Registry\Registerable;

/**
 * A Socket Container class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class SocketContainer extends BaseContainer implements Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this Container class and prepares it for usage
	 *
	 * @param	$socketResource		A valid socket resource
	 * @param	$infoInstance		An instance of a  ShareableInfo class
	 * @param	$packageData		Raw package data
	 * @return	$containerInstance	An instance of this Container class
	 */
	public static final function createSocketContainer ($socketResource, ShareableInfo $infoInstance = NULL, array $packageData = array()) {
		// Get a new instance
		$containerInstance = new SocketContainer();

		// Remove unneeded entries
		unset($packageData[NetworkPackage::PACKAGE_DATA_CONTENT]);
		unset($packageData[NetworkPackage::PACKAGE_DATA_HASH]);

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER[' . __METHOD__ . ':' . __LINE__ . ']:socketResource=' . $socketResource . ',packageData='.print_r($packageData, TRUE));

		// Is the info instance set?
		if ($infoInstance instanceof ShareableInfo) {
			// Get listener/helper from info class
			$listenerInstance = $infoInstance->getListenerInstance();
			$helperInstance = $infoInstance->getHelperInstance();

			// Debug message
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER[' . __METHOD__ . ':' . __LINE__ . ']: listenerInstance[]=' . gettype($listenerInstance));

			// Is there a listener instance set?
			if ($listenerInstance instanceof Listenable) {
				// Debug message
				//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER[' . __METHOD__ . ':' . __LINE__ . ']: Setting listenerInstance=' . $listenerInstance->__toString() . ' ...');

				// Set it here for later usage
				$containerInstance->setListenerInstance($listenerInstance);
			} elseif ($helperInstance instanceof ConnectionHelper) {
				// Debug message
				//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER[' . __METHOD__ . ':' . __LINE__ . ']: Setting helperInstance=' . $helperInstance->__toString() . ' ...');

				// Set it here for later usage
				$containerInstance->setHelperInstance($helperInstance);
			}
		} // END - if

		// Set the resource ...
		$containerInstance->setSocketResource($socketResource);

		// ... and package data
		$containerInstance->setPackageData($packageData);

		// Return the prepared instance
		return $containerInstance;
	}

	/**
	 * Checks whether the given Universal Node Locator matches with the one from package data
	 *
	 * @param	$unl		A Universal Node Locator
	 * @return	$matches	Whether $address matches with the one from package data
	 */
	public final function ifAddressMatches ($unl) {
		// Get current package data
		$packageData = $this->getPackageData();

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER[' . __METHOD__ . ':' . __LINE__ . ']: unl=' . $unl . ',packageData=' . print_r($packageData, TRUE));

		// So, does both match?
		$matches = ((isset($packageData[NetworkPackage::PACKAGE_DATA_RECIPIENT])) && ($packageData[NetworkPackage::PACKAGE_DATA_RECIPIENT] === $unl));

		// Return result
		return $matches;
	}

	/**
	 * Checks whether the given socket matches with stored
	 *
	 * @param	$socketResource		A valid socket resource
	 * @return	$matches			Whether given socket matches
	 */
	public final function ifSocketResourceMatches ($socketResource) {
		// Debug message
		/* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('SOCKET-CONTAINER[' . __METHOD__ . ':' . __LINE__ . ']: socketResource[' . gettype($socketResource) . ']=' .$socketResource . ',storedResource[' . gettype($this->getSocketResource()) . ']=' . $this->getSocketResource());

		// So, does both match?
		$matches = ((is_resource($socketResource)) && ($socketResource === $this->getSocketResource()));

		// Return result
		return $matches;
	}

}
