<?php
// Own namespace
namespace CoreFramework\Filter\Chain;

// Import framework stuff
use use CoreFramework\Registry\Registerable;
use CoreFramework\Object\BaseFrameworkSystem;

/**
 * A filter chain for pre and post filters
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FilterChain extends BaseFrameworkSystem implements Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$chainInstance	An instance of this class
	 */
	public static final function createFilterChain () {
		// Get a new instance
		$chainInstance = new FilterChain();

		// Return the prepared instance
		return $chainInstance;
	}

	/**
	 * Add a new filter
	 *
	 * @param	$filerInstance	An instance of a filter class
	 * @return	void
	 */
	public final function addFilter (Filterable $filterInstance) {
		$this->pushValueToGenericArrayKey('filters', 'generic', 'dummy', $filterInstance);
	}

	/**
	 * Add a new post-filter
	 *
	 * @param	$filerInstance	An instance of a post-filter class
	 * @return	void
	 */
	public final function addPostFilter (Filterable $filterInstance) {
		$this->pushValueToGenericArrayKey('filters', 'post', 'dummy', $filterInstance);
	}

	/**
	 * "Getter" for filters array
	 *
	 * @return	$filters	The filters array holding all filter instances
	 */
	protected function getFilters () {
		// Default is nothing found
		$filters = array();

		// Are some filters set?
		if ($this->isValidGenericArrayKey('filters', 'generic', 'dummy')) {
			// Then get them
			$filters = $this->getGenericArrayKey('filters', 'generic', 'dummy');
		} // END - if

		// Return it
		return $filters;
	}

	/**
	 * "Getter" for post-filters array
	 *
	 * @return	$filters	The filters array holding all post-filter instances
	 */
	protected function getPostFilters () {
		// Default is nothing found
		$filters = array();

		// Are some filters set?
		if ($this->isValidGenericArrayKey('filters', 'post', 'dummy')) {
			// Then get them
			$filters = $this->getGenericArrayKey('filters', 'post', 'dummy');
		} // END - if

		// Return it
		return $filters;
	}

	/**
	 * Process all added filters. Please note that filters must throw
	 * FilterChainException if they need to interrupt the filter chain.
	 *
	 * @param	$requestInstance	An instance of a request class
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	public function processFilters (Requestable $requestInstance, Responseable $responseInstance) {
		// Run all filters
		//* DEBUG */ self::createDebugInstance(__CLASS__)->debugOutput('COUNT=' . $this->countGenericArray('filters'));
		foreach ($this->getFilters() as $filterInstance) {
			// Must be an instance of Filterable
			assert($filterInstance instanceof Filterable);

			// Try to execute this filter
			try {
				//* DEBUG */ self::createDebugInstance(__CLASS__)->debugOutput('FILTER: ' . $filterInstance->__toString() . ': Processing started.');
				$filterInstance->execute($requestInstance, $responseInstance);
				//* DEBUG */ self::createDebugInstance(__CLASS__)->debugOutput('FILTER: ' . $filterInstance->__toString() . ': Processing ended.');
			} catch (FilterChainException $e) {
				// This exception can be thrown to just skip any further processing
				self::createDebugInstance(__CLASS__)->debugOutput('Failed to execute lase filter ' . $filterInstance->__toString() . ': ' . $e->getMessage());
				break;
			}
		} // END - foreach
	}

}
