<?php
// Own namespace
namespace CoreFramework\Iterator\File;

/**
 * A file iterator
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class FileIterator extends BaseIterator implements SeekableWritableFileIterator {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$pointerInstance	An instance of a Block class
	 * @return	$iteratorInstance	An instance of a Iterator class
	 */
	public final static function createFileIterator (Block $blockInstance) {
		// Get new instance
		$iteratorInstance = new FileIterator();

		// Set the instance here
		/* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d] Setting blockInstance=%s ...', __METHOD__, __LINE__, $blockInstance->__toString()));
		$iteratorInstance->setBlockInstance($blockInstance);

		// Return the prepared instance
		return $iteratorInstance;
	}

	/**
	 * Gets currently read data
	 *
	 * @return	$current	Currently read data
	 */
	public function current () {
		// Call block instance
		return $this->getBlockInstance()->current();
	}

	/**
	 * Gets current seek position ("key").
	 *
	 * @return	$key	Current key in iteration
	 */
	public function key () {
		// Return it
		return $this->getBlockInstance()->determineSeekPosition();
	}

	/**
	 * Advances to next "block" of bytes
	 *
	 * @return	void
	 */
	public function next () {
		// Call block instance
		$this->getBlockInstance()->next();
	}

	/**
	 * Rewinds to the beginning of the file
	 *
	 * @return	$status		Status of this operation
	 */
	public function rewind () {
		// Call block instance
		return $this->getBlockInstance()->rewind();
	}

	/**
	 * Checks wether the current entry is valid (not at the end of the file).
	 * This method will return TRUE if an emptied (nulled) entry has been found.
	 *
	 * @return	$isValid	Whether the next entry is valid
	 */
	public function valid () {
		// Call block instance
		return $this->getBlockInstance()->valid();
	}

	/**
	 * Seeks to given position
	 *
	 * @param	$seekPosition	Seek position in file
	 * @return	$status			Status of this operation
	 */
	public function seek ($seekPosition) {
		// Call block instance
		return $this->getBlockInstance()->seek($seekPosition);
	}

	/**
	 * Size of file stack
	 *
	 * @return	$size	Size (in bytes) of file
	 */
	public function size () {
		// Call the block object
		$size = $this->getBlockInstance()->size();

		// Return result
		return $size;
	}

	/**
	 * Reads given amount of bytes from file.
	 *
	 * @param	$bytes	Amount of bytes to read
	 * @return	$data	Data read from file
	 */
	public function read ($bytes = NULL) {
		// Call block instance
		return $this->getBlockInstance()->read($bytes);
	}

	/**
	 * Analyzes entries in index file. This will count all found (and valid)
	 * entries, mark invalid as damaged and count gaps ("fragmentation"). If
	 * only gaps are found, the file is considered as "virgin" (no entries).
	 *
	 * @return	void
	 */
	public function analyzeFile () {
		// Just call the block instance
		$this->getBlockInstance()->analyzeFile();
	}

	/**
	 * Checks whether the file header is initialized
	 *
	 * @return	$isInitialized	Whether the file header is initialized
	 */
	public function isFileHeaderInitialized () {
		// Just call the block instance
		return $this->getBlockInstance()->isFileHeaderInitialized();
	}

	/**
	 * Creates the assigned file
	 *
	 * @return	void
	 */
	public function createFileHeader () {
		// Just call the block instance
		$this->getBlockInstance()->createFileHeader();
	}

	/**
	 * Pre-allocates file (if enabled) with some space for later faster write access.
	 *
	 * @param	$type	Type of the file
	 * @return	void
	 */
	public function preAllocateFile ($type) {
		// Just call the block instance
		$this->getBlockInstance()->preAllocateFile($type);
	}

	/**
	 * Initializes counter for valid entries, arrays for damaged entries and
	 * an array for gap seek positions. If you call this method on your own,
	 * please re-analyze the file structure. So you are better to call
	 * analyzeFile() instead of this method.
	 *
	 * @return	void
	 */
	public function initCountersGapsArray () {
		// Call block instance
		$this->getBlockInstance()->initCountersGapsArray();
	}

	/**
	 * Getter for header size
	 *
	 * @return	$totalEntries	Size of file header
	 */
	public final function getHeaderSize () {
		// Call block instance
		return $this->getBlockInstance()->getHeaderSize();
	}

	/**
	 * Setter for header size
	 *
	 * @param	$headerSize		Size of file header
	 * @return	void
	 */
	public final function setHeaderSize ($headerSize) {
		// Call block instance
		$this->getBlockInstance()->setHeaderSize($headerSize);
	}

	/**
	 * Getter for header array
	 *
	 * @return	$totalEntries	Size of file header
	 */
	public final function getHeader () {
		// Call block instance
		return $this->getBlockInstance()->getHeader();
	}

	/**
	 * Setter for header
	 *
	 * @param	$header		Array for a file header
	 * @return	void
	 */
	public final function setHeader (array $header) {
		// Call block instance
		$this->getBlockInstance()->setHeader($header);
	}

	/**
	 * Updates seekPosition attribute from file to avoid to much access on file.
	 *
	 * @return	void
	 */
	public function updateSeekPosition () {
		// Call block instance
		$this->getBlockInstance()->updateSeekPosition();
	}

	/**
	 * Getter for total entries
	 *
	 * @return	$totalEntries	Total entries in this file
	 */
	public final function getCounter () {
		// Call block instance
		return $this->getBlockInstance()->getCounter();
	}

	/**
	 * "Getter" for file size
	 *
	 * @return	$fileSize	Size of currently loaded file
	 */
	public function getFileSize () {
		// Call block instance
		return $this->getBlockInstance()->getFileSize();
	}

	/**
	 * Writes data at given position
	 *
	 * @param	$seekPosition	Seek position
	 * @param	$data			Data to be written
	 * @param	$flushHeader	Whether to flush the header (default: flush)
	 * @return	void
	 */
	public function writeData ($seekPosition, $data, $flushHeader = TRUE) {
		// Call block instance
		$this->getBlockInstance()->writeData($seekPosition, $data, $flushHeader);
	}

	/**
	 * Getter for seek position
	 *
	 * @return	$seekPosition	Current seek position (stored here in object)
	 */
	public function getSeekPosition () {
		// Call block instance
		return $this->getBlockInstance()->getSeekPosition();
	}

	/**
	 * Writes given value to the file and returns a hash and gap position for it
	 *
	 * @param	$groupId	Group identifier
	 * @param	$value		Value to be added to the stack
	 * @return	$data		Hash and gap position
	 */
	public function writeValueToFile ($groupId, $value) {
		// Call block instance
		return $this->getBlockInstance()->writeValueToFile($groupId, $value);
	}

	/**
	 * Writes given raw data to the file and returns a gap position and length
	 *
	 * @param	$groupId	Group identifier
	 * @param	$hash		Hash from encoded value
	 * @param	$encoded	Encoded value to be written to the file
	 * @return	$data		Gap position and length of the raw data
	 */
	public function writeDataToFreeGap ($groupId, $hash, $encoded) {
		// Call block instance
		return $this->getBlockInstance()->writeDataToFreeGap($groupId, $hash, $encoded);
	}

	/**
	 * Searches for next suitable gap the given length of data can fit in
	 * including padding bytes.
	 *
	 * @param	$length			Length of raw data
	 * @return	$seekPosition	Found next gap's seek position
	 */
	public function searchNextGap ($length) {
		// Call block instance
		print $this->getBlockInstance()->__toString() . PHP_EOL;
		return $this->getBlockInstance()->searchNextGap($length);
	}

}
